#region Copyright (C) 2006 Torq Software.
//=============================================================================
// Copyright 2006 Torq Software - http://www.torqsoftware.com
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
// 
//     http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//=============================================================================
#endregion Copyright (C) 2006 Torq Software.

using System;
using System.ComponentModel;

namespace Torq.Foundation
{
    /// <summary>
    /// A support structure for the creation of event properties that can either take the
    /// standard .NET event implementation approach or use an approach based on the
    /// <see cref="WeakDelegate"/> functionality. The choice of which event approach to
    /// take is based on the <see cref="UseWeakDelegatesByDefault"/> property which
    /// can be set in the main method of the application. The <see cref="Delegator"/> 
    /// structure allows the application code to stay the same regardless of which approach 
    /// is chosen at runtime.
    /// </summary>
    /// <example>
    /// <code>
    /// private Delegator changedEvent;
    /// 
    /// public EventHandler ChangedEvent
    /// {
    ///     add { Delegator.Add( ref changedEvent, value ); }
    ///     remove { Delegator.Remove( ref changedEvent, value ); }
    /// }
    /// 
    /// public void OnChangedEvent( EventArgs args )
    /// {
    ///     Delegator.InvokeHandler( this, args );
    /// }
    /// </code>
    /// </example>
    public struct Delegator
    {
        internal Delegate standardDelegate;
        internal WeakDelegate weakDelegate;
        private static bool useWeakDelegatesByDefault;

        /// <summary>
        /// Gets a value indicating whether this delegator has subscribers. This can be used
        /// to check whether its worth creating event argument instances before performing
        /// an invoke action.
        /// </summary>
        public bool HasSubscribers
        {
            get { return standardDelegate != null || weakDelegate != null; }
        }

        /// <summary>
        /// Defines whether the <see cref="Delegator"/> implementation uses weak references to 
        /// ensure delegator instances don't hold subscriber target objects in memory because
        /// of an event subscription.
        /// </summary>
        public static bool UseWeakDelegatesByDefault
        {
            get { return useWeakDelegatesByDefault; }
            set { useWeakDelegatesByDefault = value; }
        }

        /// <summary>
        /// Gets a value indicating whether this <see cref="Delegator"/> instance is using
        /// the <see cref="WeakDelegate"/> functionality.
        /// </summary>
        public bool UsingWeakDelegates
        {
            get
            {
                // If a delegate approach is already in use then return an indication
                // of that use. Otherwise take the approach defined by the UseWeakDelegatesByDefault
                // static property.
                if ( standardDelegate != null )
                    return false;
                else if ( weakDelegate != null )
                    return true;
                else return UseWeakDelegatesByDefault;
            }
        }

        /// <summary>
        /// Adds the supplied subscriber delegate to the list of delegates to call when an
        /// event occurs.
        /// </summary>
        /// <param name="delegator">The event delegator.</param>
        /// <param name="subscriber">The subscriber delegate to call when an event occurs.</param>
        public static void Add( ref Delegator delegator, Delegate subscriber )
        {
            if ( delegator.UsingWeakDelegates )
                WeakDelegate.Add( ref delegator.weakDelegate, subscriber );
            else delegator.standardDelegate = Delegate.Combine( delegator.standardDelegate, subscriber );
        }

        /// <summary>
        /// Removes the supplied subscriber delegate from the list of delegates to call when an
        /// event occurs. The removal process uses the subscriber's <see cref="Delegate.Target"/>
        /// and <see cref="Delegate.Method"/> as the basis for determining which item to remove
        /// from the existing list.
        /// </summary>
        /// <param name="delegator">The event delegator.</param>
        /// <param name="subscriber">The delegate to remove from the subscriber list.</param>	    
        public static void Remove( ref Delegator delegator, Delegate subscriber )
        {
            if ( delegator.UsingWeakDelegates )
                WeakDelegate.Remove( ref delegator.weakDelegate, subscriber );
            else delegator.standardDelegate = Delegate.Remove( delegator.standardDelegate, subscriber );
        }

        /// <summary>
        /// Invokes all <see cref="EventHandler"/> instances in the subscriber list.
        /// </summary>
        /// <remarks>
        /// <para>
        /// Separate invoke methods are defined for different event handler types. This is done so
        /// that direct event handler invokes can be performed and the performance overhead associated
        /// with DynamicInvoke or Reflection calls is not encountered. Unfortunately the .NET 2.0
        /// generics implementation doesn't support delegates as generic constraints, otherwise
        /// the Invoke method could have been made into a generic method.
        /// </para>
        /// <para>
        /// A reference to the <see cref="Delegator"/> instance is passed in because the
        /// alternative <see cref="WeakDelegate"/> implementation may need to alter the delegator
        /// reference when stripping out "dead" delegate references. This allows all code that
        /// uses the <see cref="Delegator"/> infrastructure to stay the same regardless of whether
        /// the <see cref="UseWeakDelegatesByDefault"/> property is set or not.
        /// </para>
        /// </remarks>
        /// <param name="delegator">The event delegator.</param>
        /// <param name="sender">The sender object.</param>
        /// <param name="args">The <see cref="EventArgs"/> instance containing the event data.</param>
        public static void InvokeHandler( ref Delegator delegator, object sender, EventArgs args )
        {
            if ( delegator.UsingWeakDelegates )
                WeakDelegate.Invoke( ref delegator.weakDelegate, sender, args );
            else if ( delegator.standardDelegate != null )
            {
                EventHandler handler = (EventHandler) delegator.standardDelegate;
                handler( sender, args );
            }
        }

        /// <summary>
        /// Invokes all <see cref="ListChangedEventHandler"/> instances in the subscriber list.
        /// </summary>
        /// <param name="delegator">The event delegator.</param>
        /// <param name="sender">The object sender.</param>
        /// <param name="args">The <see cref="ListChangedEventArgs"/> instance containing the event data.</param>
        public static void InvokeListChangedHandler( ref Delegator delegator, object sender, ListChangedEventArgs args )
        {
            if ( delegator.UsingWeakDelegates )
                WeakDelegate.Invoke( ref delegator.weakDelegate, sender, args );
            else if ( delegator.standardDelegate != null )
            {
                ListChangedEventHandler handler = (ListChangedEventHandler) delegator.standardDelegate;
                handler( sender, args );
            }
        }

#if !DELEGATOR_EXAMPLE
        /// <summary>
        /// Invokes all <see cref="TorqPropertyEventHandler"/> instances in the subscriber list.
        /// </summary>
        /// <param name="delegator">The event delegator.</param>
        /// <param name="sender">The object sender.</param>
        /// <param name="args">The <see cref="TorqPropertyEventArgs"/> instance containing the event data.</param>
        public static void InvokePropertyEventHandler( ref Delegator delegator, object sender,
                                                       TorqPropertyEventArgs args )
        {
            if ( delegator.UsingWeakDelegates )
                WeakDelegate.Invoke( ref delegator.weakDelegate, sender, args );
            else if ( delegator.standardDelegate != null )
            {
                TorqPropertyEventHandler handler = (TorqPropertyEventHandler) delegator.standardDelegate;
                handler( sender, args );
            }
        }

        /// <summary>
        /// Invokes all <see cref="TorqListEventHandler"/> instances in the subscriber list.
        /// </summary>
        /// <param name="delegator">The event delegator.</param>
        /// <param name="sender">The object sender.</param>
        /// <param name="args">The <see cref="TorqListEventArgs"/> instance containing the event data.</param>
        public static void InvokeTorqListHandler( ref Delegator delegator, object sender, TorqListEventArgs args )
        {
            if ( delegator.UsingWeakDelegates )
                WeakDelegate.Invoke( ref delegator.weakDelegate, sender, args );
            else if ( delegator.standardDelegate != null )
            {
                TorqListEventHandler handler = (TorqListEventHandler) delegator.standardDelegate;
                handler( sender, args );
            }
        }

        /// <summary>
        /// Invokes all <see cref="TorqObjectEventHandler"/> instances in the subscriber list.
        /// </summary>
        /// <param name="delegator">The event delegator.</param>
        /// <param name="sender">The object sender.</param>
        /// <param name="args">The <see cref="TorqObjectEventArgs"/> instance containing the event data.</param>
        public static void InvokeTorqObjectHandler( ref Delegator delegator, object sender, TorqObjectEventArgs args )
        {
            if ( delegator.UsingWeakDelegates )
                WeakDelegate.Invoke( ref delegator.weakDelegate, sender, args );
            else if ( delegator.standardDelegate != null )
            {
                TorqObjectEventHandler handler = (TorqObjectEventHandler) delegator.standardDelegate;
                handler( sender, args );
            }
        }

        /// <summary>
        /// Invokes all <see cref="StringEventHandler"/> instances in the subscriber list.
        /// </summary>
        /// <param name="delegator">The event delegator.</param>
        /// <param name="sender">The object sender.</param>
        /// <param name="args">The <see cref="StringEventArgs"/> instance containing the event data.</param>
        public static void InvokeStringEventHandler( ref Delegator delegator, object sender, StringEventArgs args )
        {
            if ( delegator.UsingWeakDelegates )
                WeakDelegate.Invoke( ref delegator.weakDelegate, sender, args );
            else if ( delegator.standardDelegate != null )
            {
                StringEventHandler handler = (StringEventHandler) delegator.standardDelegate;
                handler( sender, args );
            }
        }
#endif
    }
}