#region Copyright (C) 2006 Torq Software.
//=============================================================================
// Copyright 2006 Torq Software - http://www.torqsoftware.com
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
// 
//     http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//=============================================================================
#endregion Copyright (C) 2006 Torq Software.

using System;
using System.Diagnostics;
using System.Reflection;

namespace Torq.Foundation
{
    /// <summary>
    /// Event arguments that supply information on an event subscriber object that has been
    /// garbage collected whilst still being referenced by a <see cref="WeakDelegate"/> instance.
    /// </summary>
    /// <remarks>
    /// The <see cref="StackTrace"/> property will be null unless the <see cref="WeakDelegate.TakeStackTraces"/>
    /// property has been set to true.
    /// </remarks>
    public class SubscriberGarbagedArgs : EventArgs
    {
        private MethodInfo targetMethod;
        private Type targetType;
        private StackTrace stackTrace;

        /// <summary>
        /// Initializes a new instance of the <see cref="SubscriberGarbagedArgs"/> class.
        /// </summary>
        public SubscriberGarbagedArgs( WeakDelegate weakDelegate )
        {
            targetMethod = weakDelegate.TargetMethod;
            targetType = weakDelegate.TargetType;
            stackTrace = weakDelegate.StackTrace;
        }

        /// <summary>
        /// Gets the target <see cref="MethodInfo"/> of the subscriber target that
        /// was garbage collected.
        /// </summary>
        /// <value>The method.</value>
        public MethodInfo TargetMethod
        {
            get { return targetMethod; }
        }

        /// <summary>
        /// Gets the type of the subscriber target that was garbage collected.
        /// </summary>
        /// <remarks>
        /// The subscriber instance is not available since it was garbage collected. For
        /// analysis purposes, knowing the type of the subscriber instance is helpful.
        /// </remarks>
        /// <value>The type of the target object.</value>
        public Type TargetType
        {
            get { return targetType; }
        }

        /// <summary>
        /// Gets the stack trace a the time the subscriber was added to the
        /// event delegate list. This can be null is <see cref="WeakDelegate.TakeStackTraces"/>
        /// is false.
        /// </summary>
        /// <value>The stack trace.</value>
        public StackTrace StackTrace
        {
            get { return stackTrace; }
        }
    }
}