#region Copyright (C) 2006 Torq Software.
//=============================================================================
// Copyright 2006 Torq Software - http://www.torqsoftware.com
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
// 
//     http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//=============================================================================
#endregion Copyright (C) 2006 Torq Software.

#if INCLUDE_TESTS

using System;
using System.ComponentModel;

namespace Torq.Foundation.Tests
{
    /// <summary>
    /// Subscribes to <see cref="TestEventGenerator"/> events.
    /// </summary>
    internal class TestEventSubscriber
    {
        private int listChangedCount;
        private int statusChangedCount;
        private static int totalListChangedCount;
        private static int totalStatusChangedCount;

        /// <summary>
        /// Initializes a new instance of the <see cref="TestEventSubscriber"/> class
        /// which is not subscribed to anything.
        /// </summary>        
        public TestEventSubscriber()
        {
        }
        
        /// <summary>
        /// Initializes a new instance of the <see cref="TestEventSubscriber"/> class
        /// which is subscribed to the supplied <see cref="TestEventGenerator"/>.
        /// </summary>
        /// <param name="generator">The generator of events.</param>
        public TestEventSubscriber( TestEventGenerator generator )
        {
            generator.StatusChanged += generator_StatusChanged;
            generator.ListChanged += generator_ListChanged;
        }

        /// <summary>
        /// Unsubscribes from the specified event generator.
        /// </summary>
        /// <param name="generator">The event generator.</param>
        public void Unsubscribe( TestEventGenerator generator )
        {
            generator.StatusChanged -= generator_StatusChanged;
            generator.ListChanged -= generator_ListChanged;            
        }

        /// <summary>
        /// Gets the list changed count.
        /// </summary>
        /// <value>The list changed count.</value>
        public int ListChangedCount
        {
            get { return listChangedCount; }
        }

        /// <summary>
        /// Gets the status changed count.
        /// </summary>
        /// <value>The status changed count.</value>
        public int StatusChangedCount
        {
            get { return statusChangedCount; }
        }

        /// <summary>
        /// Gets the total list changed count across all <see cref="TestEventSubscriber"/> instances.
        /// </summary>
        /// <value>The total list changed count.</value>
        public static int TotalListChangedCount
        {
            get { return totalListChangedCount; }
        }

        /// <summary>
        /// Gets the total status changed count across all <see cref="TestEventSubscriber"/> instances.
        /// </summary>
        /// <value>The total status changed count.</value>
        public static int TotalStatusChangedCount
        {
            get { return totalStatusChangedCount; }
        }

        /// <summary>
        /// Resets the total changed counts across all <see cref="TestEventSubscriber"/> instances.
        /// </summary>
        public static void ResetTotalChangedCounts()
        {
            totalStatusChangedCount = 0;
            totalListChangedCount = 0;
        }

        /// <summary>
        /// Handles the ListChanged event of the generator object.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="ListChangedEventArgs"/> instance containing the event data.</param>
        internal void generator_ListChanged( object sender, ListChangedEventArgs e )
        {
            ++listChangedCount;
            ++totalListChangedCount;
        }

        /// <summary>
        /// Handles the StatusChanged event of the generator object.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        internal void generator_StatusChanged( object sender, EventArgs e )
        {
            ++statusChangedCount;
            ++totalStatusChangedCount;
        }
    }
}

#endif