#region Copyright (C) 2006 Torq Software.
//=============================================================================
// Copyright 2006 Torq Software - http://www.torqsoftware.com
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
// 
//     http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//=============================================================================
#endregion Copyright (C) 2006 Torq Software.
#if INCLUDE_TESTS

using System;
using System.Collections.Generic;
using System.Reflection;
using NUnit.Framework;

namespace Torq.Foundation.Tests
{
    /// <summary>
    /// Performance tests for the <see cref="WeakDelegate"/> class functionality.
    /// </summary>
    [ TestFixture ]
    [ Ignore ]
    public class WeakDelegatePerformanceTest
    {
        private TestEventGenerator generator;
        private TestEventSubscriber subscriber;

        private const int InvokePerformanceTestCount = 1000000;
        private const int AddRemoveSubscriberCount = 100;
        private const int AddRemoveTestCount = 1000;

        [ SetUp ]
        public void Setup()
        {
            generator = new TestEventGenerator();
            subscriber = new TestEventSubscriber();
        }

        [ TearDown ]
        public void TearDown()
        {
            generator = null;
            subscriber = null;
        }

        [ Test ]
        public void TestInvokePerformance_UsingStandardDelegate()
        {
            EventHandler handler = new EventHandler( subscriber.generator_StatusChanged );
            DateTime start = DateTime.Now;

            for( int i = 0; i < InvokePerformanceTestCount; i++ )
                handler( this, EventArgs.Empty );

            TimeSpan elapsed = DateTime.Now.Subtract( start );
            Console.WriteLine( "Elapsed time = " + elapsed );
        }

        [Test]
        public void TestInvokePerformance_UsingStandardDelegateDynamicInvoke()
        {
            EventHandler handler = new EventHandler( subscriber.generator_StatusChanged );
            DateTime start = DateTime.Now;

            for ( int i = 0; i < InvokePerformanceTestCount; i++ )
                handler.DynamicInvoke( new object [] {this, EventArgs.Empty} );

            TimeSpan elapsed = DateTime.Now.Subtract( start );
            Console.WriteLine( "Elapsed time = " + elapsed );
        }
        
        [ Test ]
        public void TestInvokePerformance_CreatingDelegateEachTime()
        {
            EventHandler templateHandler = new EventHandler( subscriber.generator_StatusChanged );
            DateTime start = DateTime.Now;

            for( int i = 0; i < InvokePerformanceTestCount; i++ )
            {
                EventHandler handler =
                    (EventHandler) Delegate.CreateDelegate( typeof(EventHandler), subscriber, templateHandler.Method );
                handler( this, EventArgs.Empty );
            }

            TimeSpan elapsed = DateTime.Now.Subtract( start );
            Console.WriteLine( "Elapsed time = " + elapsed );
        }

        [ Test ]
        public void TestInvokePerformance_UsingReflection()
        {
            MethodInfo method = new EventHandler( subscriber.generator_StatusChanged ).Method;
            DateTime start = DateTime.Now;

            for( int i = 0; i < InvokePerformanceTestCount; i++ )
                method.Invoke( subscriber, new object [] {this, EventArgs.Empty} );

            TimeSpan elapsed = DateTime.Now.Subtract( start );
            Console.WriteLine( "Elapsed time = " + elapsed );
        }
        
        [Test]
        public void TestAddRemovePerformance_UsingStandardEvents()
        {
            List<TestEventSubscriber> subscribers = GetTestSubscriberList();
            DateTime start = DateTime.Now;

            for( int i=0; i<AddRemoveTestCount; i++ )
            {
                foreach( TestEventSubscriber s in subscribers )
                    generator.StandardStatusChange += s.generator_StatusChanged;

                foreach ( TestEventSubscriber s in subscribers )
                    generator.StandardStatusChange -= s.generator_StatusChanged;
            }
            
            TimeSpan elapsed = DateTime.Now.Subtract( start );
            Console.WriteLine( "Elapsed time = " + elapsed );
        }

        [Test]
        public void TestAddRemovePerformance_UsingDelegatorWithWeakDelegates()
        {
            Delegator.UseWeakDelegatesByDefault = true;
            AddRemovePerformanceUsingDelegator();
        }

        [Test]
        public void TestAddRemovePerformance_UsingDelegatorWithStandardDelegates()
        {
            Delegator.UseWeakDelegatesByDefault = false;
            AddRemovePerformanceUsingDelegator();
        }
        
        private void AddRemovePerformanceUsingDelegator()
        {
            List<TestEventSubscriber> subscribers = GetTestSubscriberList();
            DateTime start = DateTime.Now;

            for ( int i=0; i<AddRemoveTestCount; i++ )
            {
                foreach ( TestEventSubscriber s in subscribers )
                    generator.StatusChanged += s.generator_StatusChanged;

                foreach ( TestEventSubscriber s in subscribers )
                    generator.StatusChanged -= s.generator_StatusChanged;
            }

            TimeSpan elapsed = DateTime.Now.Subtract( start );
            Console.WriteLine( "Elapsed time = " + elapsed );            
        }
        
        /// <summary>
        /// Gets a subscriber list.
        /// </summary>
        /// <returns>A list of test subscribers.</returns>
        List<TestEventSubscriber> GetTestSubscriberList()
        {
            List<TestEventSubscriber> list = new List<TestEventSubscriber>( AddRemoveSubscriberCount );
            for ( int i=0; i<AddRemoveSubscriberCount; i++ )
            {
                TestEventSubscriber sub = new TestEventSubscriber();
                list.Add( sub );
            }
            
            return list;
        }
    }
}

#endif