#region Copyright (C) 2006 Torq Software.
//=============================================================================
// Copyright 2006 Torq Software - http://www.torqsoftware.com
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
// 
//     http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//=============================================================================
#endregion Copyright (C) 2006 Torq Software.

#if INCLUDE_TESTS

using System;
using System.ComponentModel;
using NUnit.Framework;

namespace Torq.Foundation.Tests
{
    /// <summary>
    /// Tests the <see cref="WeakDelegate"/> class functionality.
    /// </summary>
    [ TestFixture ]
    public class WeakDelegateTest
    {
        private bool wasUsingWeakDelegatesByDefault;
        private TestEventGenerator generator;
        private TestEventSubscriber subscriber1;
        private TestEventSubscriber subscriber2;
        private TestEventSubscriber subscriber3;
        private TestEventSubscriber subscriber4;
        private int subscriberGarbagedCount;
        private static int handleEventCount;
        private static int handleListChangedEventCount;

        [ SetUp ]
        public void Setup()
        {
            wasUsingWeakDelegatesByDefault = Delegator.UseWeakDelegatesByDefault;
            Delegator.UseWeakDelegatesByDefault = true;
            subscriberGarbagedCount = 0;

            TestEventSubscriber.ResetTotalChangedCounts();
            generator = new TestEventGenerator();
            subscriber1 = new TestEventSubscriber( generator );
            subscriber2 = new TestEventSubscriber( generator );
            subscriber3 = new TestEventSubscriber( generator );
            subscriber4 = new TestEventSubscriber( generator );

            WeakDelegate.SubscriberGarbaged += WeakDelegate_SubscriberGarbaged;
        }

        void WeakDelegate_SubscriberGarbaged( object sender, SubscriberGarbagedArgs e )
        {
            ++subscriberGarbagedCount;
        }

        [ TearDown ]
        public void TearDown()
        {
            generator = null;
            subscriber1 = null;
            subscriber2 = null;
            subscriber3 = null;
            subscriber4 = null;
            Delegator.UseWeakDelegatesByDefault = wasUsingWeakDelegatesByDefault;
            WeakDelegate.SubscriberGarbaged -= WeakDelegate_SubscriberGarbaged;            
        }

        [ Test ]
        public void TestRaiseEventHandler()
        {
            Assert.AreEqual( 0, subscriber1.StatusChangedCount );
            Assert.AreEqual( 0, subscriber2.StatusChangedCount );
            Assert.AreEqual( 0, subscriber3.StatusChangedCount );
            Assert.AreEqual( 0, subscriber4.StatusChangedCount );

            generator.RaiseStatusChanged();

            Assert.AreEqual( 1, subscriber1.StatusChangedCount );
            Assert.AreEqual( 1, subscriber2.StatusChangedCount );
            Assert.AreEqual( 1, subscriber3.StatusChangedCount );
            Assert.AreEqual( 1, subscriber4.StatusChangedCount );
        }

        [ Test ]
        public void TestRaiseListChangedEventHandler()
        {
            Assert.AreEqual( 0, subscriber1.ListChangedCount );
            Assert.AreEqual( 0, subscriber2.ListChangedCount );
            Assert.AreEqual( 0, subscriber3.ListChangedCount );
            Assert.AreEqual( 0, subscriber4.ListChangedCount );

            generator.RaiseListChanged( ListChangedType.ItemAdded, 1, 0 );

            Assert.AreEqual( 1, subscriber1.ListChangedCount );
            Assert.AreEqual( 1, subscriber2.ListChangedCount );
            Assert.AreEqual( 1, subscriber3.ListChangedCount );
            Assert.AreEqual( 1, subscriber4.ListChangedCount );
        }

        [ Test ]
        public void TestRaiseStaticEventHandler()
        {
            TestEventGenerator testGenerator = new TestEventGenerator();
            testGenerator.StatusChanged += HandleEvent;
            int expectedCount = handleEventCount + 1;
            testGenerator.RaiseStatusChanged();
            Assert.AreEqual( expectedCount, handleEventCount );
        }

        private static void HandleEvent( object sender, EventArgs args )
        {
            ++handleEventCount;
        }

        [ Test ]
        public void TestRaiseStaticListChangedEventHandler()
        {
            TestEventGenerator testGenerator = new TestEventGenerator();
            testGenerator.ListChanged += HandleListChangedEvent;
            int expectedCount = handleListChangedEventCount + 1;
            testGenerator.RaiseListChanged( ListChangedType.ItemDeleted, 2, 0 );
            Assert.AreEqual( expectedCount, handleListChangedEventCount );
        }

        private static void HandleListChangedEvent( object sender, ListChangedEventArgs args )
        {
            ++handleListChangedEventCount;
        }

        [Test]
        public void TestRaiseMultipleEventHandler()
        {
            Assert.AreEqual( 0, subscriber1.StatusChangedCount );
            Assert.AreEqual( 0, subscriber2.StatusChangedCount );
            Assert.AreEqual( 0, subscriber3.StatusChangedCount );
            Assert.AreEqual( 0, subscriber4.StatusChangedCount );

            const int RaisedEventCount = 8;

            for ( int i=0; i<RaisedEventCount; i++ )
                generator.RaiseStatusChanged();

            Assert.AreEqual( RaisedEventCount, subscriber1.StatusChangedCount );
            Assert.AreEqual( RaisedEventCount, subscriber2.StatusChangedCount );
            Assert.AreEqual( RaisedEventCount, subscriber3.StatusChangedCount );
            Assert.AreEqual( RaisedEventCount, subscriber4.StatusChangedCount );
        }
        
        [ Test ]
        public void TestGCFirstEventHandler()
        {
            subscriber4 = null;
            WaitForGarbageCollection();
            generator.RaiseStatusChanged();
            Assert.AreEqual( 3, TestEventSubscriber.TotalStatusChangedCount );
            Assert.AreEqual( 1, subscriberGarbagedCount );            
        }

        [ Test ]
        public void TestGCMiddleEventHandler()
        {
            subscriber2 = null;
            subscriber3 = null;
            WaitForGarbageCollection();
            generator.RaiseStatusChanged();
            Assert.AreEqual( 2, TestEventSubscriber.TotalStatusChangedCount );
            Assert.AreEqual( 2, subscriberGarbagedCount );
        }

        [ Test ]
        public void TestGCLastEventHandler()
        {
            subscriber1 = null;
            WaitForGarbageCollection();
            generator.RaiseStatusChanged();
            Assert.AreEqual( 3, TestEventSubscriber.TotalStatusChangedCount );
            Assert.AreEqual( 1, subscriberGarbagedCount );            
        }

        [ Test ]
        public void TestSubscriptionRemoval()
        {
            subscriber1.Unsubscribe( generator );
            subscriber2.Unsubscribe( generator );

            generator.RaiseStatusChanged();
            Assert.AreEqual( 0, subscriber1.StatusChangedCount );
            Assert.AreEqual( 0, subscriber2.StatusChangedCount );
            Assert.AreEqual( 1, subscriber3.StatusChangedCount );
            Assert.AreEqual( 1, subscriber4.StatusChangedCount );
            Assert.AreEqual( 0, subscriberGarbagedCount );            
        }

        private void WaitForGarbageCollection()
        {
            GC.Collect();
            GC.WaitForPendingFinalizers();
        }
    }
}

#endif