#region Copyright (C) 2006 Torq Software.
//=============================================================================
// Copyright 2006 Torq Software - http://www.torqsoftware.com
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
// 
//     http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//=============================================================================
#endregion Copyright (C) 2006 Torq Software.

using System;
using System.Diagnostics;
using System.Reflection;

namespace Torq.Foundation
{
    /// <summary>
    /// A class that supports the creation of event properties that weakly reference any
    /// objects that subscribe to the event. See <see cref="WeakReference"/> for a description
    /// of the standard weak referencing functionality built into the .NET Framework. A weak
    /// referencing delegate ensures that subscriber objects are not held in memory simply
    /// because of an entry in an event's subscriber list.
    /// </summary>
    /// <remarks>
    /// <list type="bullet">
    /// <item>Only delegates that derive from <see cref="EventHandler"/> are supported.</item>
    /// <item>Only event arguments that derive from <see cref="EventArgs"/> are supported.</item>
    /// <item>The class is not thread safe. It assumes execution within a single thread.</item>
    /// <item>
    /// When a <see cref="WeakDelegate"/> is initialized with a <see cref="Delegate"/> supplied by 
    /// a subscriber, no reference to the <see cref="Delegate"/> is kept. Instead the
    /// delegate's <see cref="Delegate.Method"/> is stored. In addition the <see cref="Delegate.Target"/> 
    /// is placed within a <see cref="WeakReference"/>.
    /// </item>
    /// <item>
    /// The <see cref="TotalAddCount"/>, <see cref="TotalRemoveCount"/>, <see cref="TotalRaiseEventCount"/>
    /// and <see cref="TotalMethodCallEventCount"/> properties are all updated when this class is compiled
    /// in DEBUG mode. These properties allow a measurement of how often events are added, removed and
    /// invoked during a program execution. They can be used decide whether the <see cref="WeakDelegate"/> 
    /// approach can be practically used from a performance perspective in an application scenario.
    /// </item>
    /// </list>
    /// </remarks>
    public class WeakDelegate : WeakReference
    {
        private MethodInfo targetMethod;
        private Type targetType;
        private WeakDelegate next;
        private StackTrace stackTrace;

        private static bool takeStackTraces;
        private static int totalAddCount;
        private static int totalRemoveCount;
        private static int totalRaiseEventCount;
        private static int totalMethodCallEventCount;

        /// <summary>
        /// Initializes a new instance of the <see cref="WeakDelegate"/> class.
        /// </summary>
        /// <param name="subscriber">The delegate supplied by the subscriber.</param>
        private WeakDelegate( Delegate subscriber )
            : base( subscriber.Target )
        {
            SetSubscriberInfo( subscriber );
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="WeakDelegate"/> class to be used as
        /// the head object in a linked list of instances.
        /// </summary>
        /// <param name="weakDelegate">
        /// The previous head object in a linked list of <see cref="WeakDelegate"/> instances.
        /// </param>
        /// <param name="subscriber">The delegate supplied by the subscriber.</param>
        private WeakDelegate( WeakDelegate weakDelegate, Delegate subscriber )
            : base( subscriber.Target )
        {
            SetSubscriberInfo( subscriber );
            next = weakDelegate;
        }

        /// <summary>
        /// Sets the subscriber information but excluding the <see cref="Delegate.Target"/> to
        /// ensure no strong reference links this object to the subscriber target object.
        /// </summary>
        /// <param name="subscriber">The subscriber.</param>
        private void SetSubscriberInfo( Delegate subscriber )
        {
            targetMethod = subscriber.Method;
            if ( subscriber.Target != null )
                targetType = subscriber.Target.GetType();     
            
            if ( TakeStackTraces )
                stackTrace = new StackTrace();
        }

        /// <summary>
        /// Gets whether the delegate method is static.
        /// </summary>
        public bool IsStatic
        {
            get { return TargetMethod.IsStatic; }
        }

        /// <summary>
        /// Adds the specified <see cref="Delegate"/> supplied by a subscriber to the list
        /// of subscribers.
        /// </summary>
        /// <param name="weakDelegate">
        /// The weak delegate instance that was previously at the head of a linked list of delegates.
        /// </param>
        /// <param name="subscriber">A delegate supplied by a subscriber.</param>
        /// <returns>The new head of a linked list of <see cref="WeakDelegate"/> instances.</returns>
        public static void Add( ref WeakDelegate weakDelegate, Delegate subscriber )
        {
            StripDeadReferences( ref weakDelegate );

            if ( subscriber != null )
            {
                if ( weakDelegate == null )
                    weakDelegate = new WeakDelegate( subscriber );
                else weakDelegate = new WeakDelegate( weakDelegate, subscriber );
            }

            ++totalAddCount;
        }

        /// <summary>
        /// Removes the specified <see cref="Delegate"/> defined by a subscriber from a
        /// linked list defined by the supplied weak delegate instance. The linked list
        /// is searched for any existing <see cref="WeakDelegate"/> that matches the supplied
        /// <see cref="Delegate"/>. Any matches for both the target and method are removed.
        /// </summary>
        /// <param name="weakDelegate">
        /// The weak delegate currently at the head of a linked list of weak delegates.
        /// </param>
        /// <param name="removeDelegate">
        /// A delegate supplied by a subscriber which is to be removed from the existing linked list.
        /// </param>
        /// <returns>
        /// The head of a linked list of <see cref="WeakDelegate"/> instances which doesn't contain
        /// the subscriber delegate.
        /// </returns>
        public static void Remove( ref WeakDelegate weakDelegate, Delegate removeDelegate )
        {
            StripDeadReferencesAndRemoveDelegate( ref weakDelegate, removeDelegate );
            ++totalRemoveCount;
        }

        /// <summary>
        /// Invokes the delegate on all subscribers.
        /// </summary>
        /// <param name="sender">The sender of the event.</param>
        /// <param name="args">The event arguments.</param>
        public static void Invoke( ref WeakDelegate weakDelegate, object sender, EventArgs args )
        {
            StripDeadReferences( ref weakDelegate );
            WeakDelegate current = weakDelegate;

            while ( current != null )
            {
                current.InvokeEventHandler( sender, args );
                current = current.next;
            }

            ++totalRaiseEventCount;
        }

        /// <summary>
        /// Invokes the delegate method associated with this weak delegate instance. Note that
        /// the actual <see cref="Delegate"/> instance that performs the invoke is created here
        /// and used in the moment so that no hard reference to the target object is kept within
        /// this weak delegate instance.
        /// </summary>
        /// <param name="sender">The sender object.</param>
        /// <param name="args">The event arguments.</param>
        private void InvokeEventHandler( object sender, EventArgs args )
        {
            if ( !HasTargetBeenGarbaged )
                TargetMethod.Invoke( Target, new object [] {sender, args} );

            ++totalMethodCallEventCount;
        }

        /// <summary>
        /// Strips any references to target objects that have been garbage collected from
        /// the linked list of weak delegate instances.
        /// </summary>
        /// <param name="headOfList">The head of a linked list of weak delegate instances.</param>
        public static void StripDeadReferences( ref WeakDelegate headOfList )
        {
            StripDeadReferencesAndRemoveDelegate( ref headOfList, null );
        }
        
        /// <summary>
        /// Strips any references to target objects that have been garbage collected from
        /// the linked list of weak delegate instances.
        /// </summary>
        /// <param name="headOfList">The head of a linked list of weak delegate instances.</param>
        private static void StripDeadReferencesAndRemoveDelegate( ref WeakDelegate headOfList, Delegate removalDelegate )
        {
            WeakDelegate current = headOfList;
            WeakDelegate previous = null;

            while ( current != null )
            {
                if ( current.HasTargetBeenGarbaged || current.MatchesDelegate( removalDelegate ) )
                {
                    if ( previous != null )
                    {
                        // A dead reference has been found in the middle of the linked list.
                        previous.next = current.next;
                    }
                    else
                    {
                        // A dead reference has been found at the head of the linked list.
                        headOfList = current.next;
                    }

                    if ( SubscriberGarbaged != null && current.HasTargetBeenGarbaged )
                        SubscriberGarbaged( current, new SubscriberGarbagedArgs( current ) );
                }
                else
                {
                    // Keep a note of the previous item in the linked list in case a subsequent
                    // item has to be removed.
                    previous = current;
                }

                // Iterate to the next item in the linked list.                
                current = current.next;
            }
        }

        /// <summary>
        /// Occurs when a <see cref="WeakDelegate"/> target object (subscriber) has been
        /// garbage collected. This implies that if the standard event handler approach
        /// was being used, the subscriber would have been kept in memory because of its
        /// listing in an event subscriber list. Knowing that this has occurred and
        /// the subscriber type/method can help in resolving memory leak issues when
        /// the standard event handler approach is being used.
        /// </summary>
        public static event EventHandler<SubscriberGarbagedArgs> SubscriberGarbaged;
        
        /// <summary>
        /// Gets a value indicating whether this the target of this <see cref="WeakDelegate"/>
        /// is an object instance (not static) has been garbage collected.
        /// </summary>
        private bool HasTargetBeenGarbaged
        {
            get { return !IsStatic && !IsAlive; }
        }

        /// <summary>
        /// Gets the total number of event call backs added to <see cref="WeakDelegate"/> instances.
        /// </summary>
        public static int TotalAddCount
        {
            get { return totalAddCount; }
        }

        /// <summary>
        /// Gets the total number of event call backs removed from <see cref="WeakDelegate"/> instances.
        /// </summary>
        public static int TotalRemoveCount
        {
            get { return totalRemoveCount; }
        }

        /// <summary>
        /// Gets the total number of events raised by <see cref="WeakDelegate"/> based event implementations.
        /// </summary>
        /// <value>The raise event count.</value>
        public static int TotalRaiseEventCount
        {
            get { return totalRaiseEventCount; }
        }

        /// <summary>
        /// Gets the total number of method calls initiated by <see cref="WeakDelegate"/> instances
        /// as a result of events being raised.
        /// </summary>
        /// <value>The total method call event count.</value>
        public static int TotalMethodCallEventCount
        {
            get { return totalMethodCallEventCount; }
        }

        /// <summary>
        /// Gets the target method for this <see cref="WeakDelegate"/>.
        /// </summary>
        /// <value>The target method.</value>
        public MethodInfo TargetMethod
        {
            get { return targetMethod; }
        }

        /// <summary>
        /// Gets the type of the target object for this <see cref="WeakDelegate"/>.
        /// </summary>
        /// <value>The type of the target object.</value>
        public Type TargetType
        {
            get { return targetType; }
        }

        /// <summary>
        /// Gets or sets a value indicating whether weak delegates take a note of the
        /// stack trace when a delegate is added to an event subscription list.
        /// </summary>
        public static bool TakeStackTraces
        {
            get { return takeStackTraces; }
            set { takeStackTraces = value; }
        }

        /// <summary>
        /// Gets the stack trace that was current at the time this weak delegate was constructed.
        /// </summary>
        /// <value>The stack trace.</value>
        public StackTrace StackTrace
        {
            get { return stackTrace; }
        }

        /// <summary>
        /// Determines whether this <see cref="WeakDelegate"/> is referencing the same target and
        /// method as the supplied <see cref="Delegate"/>.
        /// </summary>
        /// <param name="d">The comparison delegate.</param>
        /// <returns></returns>
        private bool MatchesDelegate( Delegate d )
        {
            if ( d != null && d.Method == TargetMethod )
            {
                if ( IsStatic )
                    return d.Target == null;
                else
                    return d.Target != null && Target == d.Target;
            }
            else return false;
        }
    }
}